// Menu.h for Openbox
// Copyright (c) 2002 - 2002 Ben Jansens (ben@orodu.net)
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

#ifndef   __Menu_hh
#define   __Menu_hh

#include <X11/Xlib.h>
#include "Geometry.h"

class MenuItem;
class Openbox;
class BImageControl;
class BScreen;

#include <string>
#include <vector>
typedef vector<MenuItem *> itemList;

class Menu {
  Openbox &_openbox;
  Display *_display;
  Menu *_parent;
  BImageControl &_image_ctrl;
  BScreen &_screen;

  bool _moving, _visible, _movable, _torn, _internal_menu, _title_vis, _shifted,
    _hide_tree;
  int _which_sub, _which_press, _which_sbl, _alignment;

  Pixmap _frame_pixmap, _title_pixmap, _hilite_pixmap, _sel_pixmap;
  Window _window, _frame, _title;

  std::string _label;
  Rect _area;
  Point _move;
  Point _shift;
  int _sublevels, _persub, _minsub;
  Size _item_sz;
  unsigned int _title_h, _frame_h;
  
  itemList _items;

protected:
  inline bool titleVisible() const { return _title_vis; }
  inline void setTitleVisible(bool b) { _title_vis = b; }

  inline bool movable() const { return _movable; }
  inline void setMovable(bool b) { _movable = b; }

  inline bool hideTree() const { return _hide_tree; }
  inline void setHideTree(bool b) { _hide_tree = b; }

  inline int minSublevels() const { return _minsub; }
  inline void setMinSublevels(int i) { _minsub = i; }

  inline bool internal() const { return _internal_menu; }
  inline void setInternal(bool b) { _internal_menu = b; }
  
  inline int alignment() const { return _alignment; }
  inline void setAlignment(int a) { _alignment = a; }
  
  virtual void selected(int, int) = 0;
  
  virtual void drawItem(int, bool = false, bool = false, int = -1, int = -1,
                        unsigned int = 0, unsigned int = 0);
  virtual void drawSubmenu(int);
  virtual void redrawTitle();
  virtual void internal_hide();

  int add(std::string &, int = 0, std::string & = "", int = -1);
  int remove(int);

public:
  Menu(BScreen &);
  virtual Menu();

  inline bool torn() const { return _torn; }
  inline void setTorn(bool b) { _torn = true; }
  
  inline bool visible() const { return _visible; }
  inline BScreen &screen() { return _screen; }
  inline window() const { return window; }
  inline int items() const { return _items.size(); }

  inline const std::string &label() const { return _label; }
  inline void setLabel(std::string &s) { _label = s; }

  inline Rect area() const { return _area; }
  inline unsigned int titleHeight() const { return title_h; }
  
  inline int currentSubmenu() const { return which_sub; }

  inline void removeParent() { if (_internal_menu) parent = NULL; }

  bool hasSubmenu(int) const;
  
  bool isSelected(int) const;
  void setSelected(int, bool = true);
  
  bool isEnabled(int) const;
  void setEnabled(int, bool = true);
  
  void buttonPressEvent(XButtonEvent *);
  void buttonReleaseEvent(XButtonEvent *);
  void motionNotifyEvent(XMotionEvent *);
  void enterNotifyEvent(XCrossingEvent *);
  void leaveNotifyEvent(XCrossingEvent *);
  void exposeEvent(XExposeEvent *);
  void move(int, int);
  virtual void show(bool = true);

  reconfigure();
  update();
  
  enum { AlignDontCare = 1, AlignTop, AlignBottom };
  enum { Right = 1, Left };
  enum { Empty = 0, Square, Triangle, Diamond };
};

class MenuItem {
  Menu *_submenu;
  std::string _label;
  std::string _exec;
  void *_function;
  bool enabled, selected;

  friend class Menu;

public:
  MenuItem(std::string &label, void *funtion, std::string &exec = "") :
    _submenu(NULL), _label(label), _exec(exec), _function(function),
    enabled(true), selected(false) {}
  MenuItem(std::string &label, Menu *submenu) : 
    _submenu(submenu), _label(label), _exec(""), _function(NULL),
    enabled(true), selected(false) {}
  MenuItem(std::string &label, int fp) :
    _submenu(NULL), _label(label), _exec(""), _function(function),
    enabled(true), selected(false) {}

  inline const std::string &label() const { return _label; }
  inline const std::string &exec() const { return _exec; }
  inline void *function() const { return _function; }
  inline Menu *submenu() { return _submenu; }

  inline bool enabled() const { return _enabled; }
  inline void setEnabled(bool e) { _enabled = e; }
  
  inline bool selected() const { return _selected; }
  inline void setSelected(bool s) { _selected = s; }
};

#endif // __Menu_hh
